"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getLoggedInTestClient = exports.stopDeepstreamServer = exports.getStartedDeepstreamServer = exports.provide = void 0;

var _server = require("@deepstream/server");

var _client = require("@deepstream/client");

var _ctrlUtils = require("../ctrl-utils");

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance"); }

function _iterableToArrayLimit(arr, i) { if (!(Symbol.iterator in Object(arr) || Object.prototype.toString.call(arr) === "[object Arguments]")) { return; } var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

var provide =
/*#__PURE__*/
function () {
  var _ref = _asyncToGenerator(
  /*#__PURE__*/
  regeneratorRuntime.mark(function _callee(dsClient, name, callback) {
    var interval,
        finished,
        internalCallback,
        _args = arguments;
    return regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            interval = _args.length > 3 && _args[3] !== undefined ? _args[3] : 20;
            finished = false;

            internalCallback = function internalCallback(data, response) {
              if (finished) callback(data, response);else {
                response.send();
                finished = true;
              }
            };

            dsClient.rpc.provide(name, internalCallback);
            return _context.abrupt("return", (0, _ctrlUtils.pollUntil)(function () {
              return finished;
            }, function () {
              return dsClient.rpc.make(name);
            }, interval // 5000,
            // true,
            ));

          case 5:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));

  return function provide(_x, _x2, _x3) {
    return _ref.apply(this, arguments);
  };
}();

exports.provide = provide;
var deepstreamServerConfs = {
  empty: {},
  withAuth: {
    auth: [{
      type: 'http',
      options: {
        endpointUrl: 'http://localhost:8000/authenticate',
        permittedStatusCodes: [200],
        requestTimeout: 8000,
        retryStatusCodes: [404, 504],
        retryAttempts: 3,
        retryInterval: 5000
      }
    }]
  }
};

var getStartedDeepstreamServer = function getStartedDeepstreamServer() {
  var confKey = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 'empty';
  var dss = new _server.Deepstream(deepstreamServerConfs[confKey]);

  var _triggerPromise = (0, _ctrlUtils.triggerPromise)(),
      _triggerPromise2 = _slicedToArray(_triggerPromise, 2),
      serverStartPromise = _triggerPromise2[0],
      serverStartResolve = _triggerPromise2[1];

  dss.on('started', serverStartResolve);
  dss.start();
  return [(0, _ctrlUtils.asyncWithTimeout)(serverStartPromise), dss];
};

exports.getStartedDeepstreamServer = getStartedDeepstreamServer;

var stopDeepstreamServer = function stopDeepstreamServer(server) {
  var _triggerPromise3 = (0, _ctrlUtils.triggerPromise)(),
      _triggerPromise4 = _slicedToArray(_triggerPromise3, 2),
      serverStopPromise = _triggerPromise4[0],
      serverStopResolve = _triggerPromise4[1];

  server.on('stopped', serverStopResolve);
  server.services.httpService.server.shutdown(); // Release internal socket handle

  server.stop();
  return (0, _ctrlUtils.asyncWithTimeout)(serverStopPromise);
};

exports.stopDeepstreamServer = stopDeepstreamServer;

var getLoggedInTestClient =
/*#__PURE__*/
function () {
  var _ref3 = _asyncToGenerator(
  /*#__PURE__*/
  regeneratorRuntime.mark(function _callee2(_ref2) {
    var _ref2$credentials, credentials, _ref2$options, options, doLog, _ref2$address, address, defaultOptions, testClient, _triggerPromise5, _triggerPromise6, clientConnectedPromise, clientConnected;

    return regeneratorRuntime.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            _ref2$credentials = _ref2.credentials, credentials = _ref2$credentials === void 0 ? {
              id: 'testClient'
            } : _ref2$credentials, _ref2$options = _ref2.options, options = _ref2$options === void 0 ? {} : _ref2$options, doLog = _ref2.doLog, _ref2$address = _ref2.address, address = _ref2$address === void 0 ? 'localhost:6020' : _ref2$address;
            defaultOptions = {
              reconnectIntervalIncrement: 10,
              maxReconnectAttempts: Infinity
            };
            testClient = new _client.DeepstreamClient(address, _objectSpread({}, defaultOptions, {}, options));
            _triggerPromise5 = (0, _ctrlUtils.triggerPromise)(), _triggerPromise6 = _slicedToArray(_triggerPromise5, 2), clientConnectedPromise = _triggerPromise6[0], clientConnected = _triggerPromise6[1];
            testClient.on('connectionStateChanged', function (cState) {
              if (doLog) console.log('testClient connection state changed:', cState);
              if (cState === 'OPEN') clientConnected();
            });
            testClient.on('error', function (e) {
              if (doLog) console.log('Test-client Error:', e.message);
            });
            _context2.next = 8;
            return testClient.login(credentials);

          case 8:
            _context2.next = 10;
            return (0, _ctrlUtils.asyncWithTimeout)(clientConnectedPromise);

          case 10:
            return _context2.abrupt("return", testClient);

          case 11:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2);
  }));

  return function getLoggedInTestClient(_x4) {
    return _ref3.apply(this, arguments);
  };
}();

exports.getLoggedInTestClient = getLoggedInTestClient;